
function msgPlayers(msg)
    msgLog(msg)
	for i, player in pairs(game.players) do
		if player.connected then
			player.print(msg);
		end
	end
end

function msgLog(msg)
    game.write_file("GhostFinder/found.txt", msg, true);
end

function clearLog()
    game.write_file("GhostFinder/found.txt", "");
end

function findGhostsInArea(surface, left, top, right, bottom)
    local entityGhost = surface.count_entities_filtered({type="entity-ghost", area={{left,top},{right,bottom}}});
    local tileGhost = surface.count_entities_filtered({type="tile-ghost", area={{left,top},{right,bottom}}});

    return entityGhost + tileGhost, entityGhost, tileGhost;
end

function showGhostToPlayer(ghost, position, player)
	player.surface.create_entity({name = ghost, position = position});
end

function showGhostToForce(ghost, position, force)
	if #force.players == 0 then
		return;
	end
	for i, player in pairs(force.players) do
		if player.connected then
			showGhostToPlayer(ghost, position, player);
		end
	end
end

local curSurface = 1;
local gameArea = {};
local areaSize = 32;

local ghostCountTotal = 0;
local ghostCountSurface = 0;
local doScan = false;

function onTick(event)
    local numSurfaces = #game.surfaces;

    if game.tick % 3600 == 0 then
        ghostCountTotal = 0;
        ghostCountSurface = 0;
        doScan = true;
    end

    if doScan then    
        msgPlayers("Starting scan for ghosts....");
        local left=0;
        local right=0;
        local top=0;
        local bottom=0;

        for chunk in game.surfaces[curSurface].get_chunks() do
            left = math.min(left, chunk.x);
            top = math.min(top, chunk.y);
            right = math.max(right, chunk.x);
            bottom = math.max(bottom, chunk.y);
        end

        gameArea.left = left * 32;
        gameArea.right = right * 32;
        gameArea.top = top * 32;
        gameArea.bottom = bottom * 32;

        for areaX=gameArea.left, gameArea.right, areaSize do
            for areaY=gameArea.top, gameArea.bottom, areaSize do
                total, entities, tiles = findGhostsInArea(game.surfaces[curSurface], areaX, areaY, areaX + areaSize, areaY + areaSize);
                ghostCountTotal = ghostCountTotal + total;
                ghostCountSurface = ghostCountSurface + total
                if total > 0 then
                    local areaCenterX = (areaX + (areaSize) / 2);
                    local areaCenterY = (areaY + (areaSize) / 2);
                    msgLog("Found " .. tostring(total) .. " ghosts around [" .. areaCenterX .. ", " .. areaCenterY .. "].");
                    msgLog(".    " .. tostring(entities) .. " are entites ghosts.");
                    msgLog(".    " .. tostring(tiles) .. " are tiles ghosts.");
                    game.take_screenshot({
                        surface = game.surfaces[curSurface],
                        position = { areaCenterX, areaCenterY },
                        resolution = { 1920 , 1080 },
                        zoom = .70,
                        path = "GhostFinder/" .. areaCenterX .. " X " .. areaCenterY .. ".png",
                        show_entity_info = fase
                    });
                    --showGhostToPlayer("GhostFinder", {areaCenterX, areaCenterY}, game.players[1]);
                end
            end
        end
    
        if ghostCountSurface > 0 then
            msgPlayers("Found " .. tostring(ghostCountSurface) .. " ghosts total on surface [" .. game.surfaces[curSurface].name .. "]." );
            ghostCountSurface = 0;
        end

        curSurface = curSurface + 1;
        if curSurface > numSurfaces then
            curSurface = 1;
            doScan = false;
            if ghostCountTotal > 0 then
                msgPlayers("Found " .. tostring(ghostCountTotal) .. " ghosts in total." );
                if ghostCountTotal > 600 then
                    local ghostOverage = ghostCountTotal - 600;
                    msgPlayers("You are over Factorios ghost job limit threshold by " .. tostring(ghostOverage) .. " ghosts.");
                    if ghostOverage > 35 then
                        msgPlayers("You have also exceeded my estimated threshold on when the bots will have noticeably slowed down.");
                    end
                end
            end
        end
    end
end

script.on_event(defines.events.on_tick, onTick);
